'use strict';

var _parseCSS = require('./parseCSS');

var _parseCSS2 = _interopRequireDefault(_parseCSS);

var _babylon = require('babylon');

var babylon = _interopRequireWildcard(_babylon);

var _touch = require('touch');

var _touch2 = _interopRequireDefault(_touch);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _hash4 = require('./hash');

var _hash5 = _interopRequireDefault(_hash4);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function getName(str) {
  var regex = /name\s*:\s*([A-Za-z0-9\-_]+)\s*/gm;
  var match = regex.exec(str);
  if (match) {
    return match[1];
  }
}

function parser(path) {
  var code = path.hub.file.code;
  var strs = path.node.quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash5.default)([].concat(_toConsumableArray(strs))); // todo - add current filename?
  var name = getName(strs.join('xxx')) || 'css';

  var stubs = path.node.quasi.expressions.map(function (x) {
    return code.substring(x.start, x.end);
  });
  var ctr = 0;

  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== stubs.length) {
      // todo - test for preceding @apply
      var applyMatch = /@apply\s*$/gm.exec(str);
      if (applyMatch) {
        arr.push('--' + name + '-' + hash + '-' + i);
      } else arr.push('var(--' + name + '-' + hash + '-' + i + ')');
    }
    return arr;
  }, []).join('').trim();
  var rules = (0, _parseCSS2.default)('.' + name + '-' + hash + ' { ' + src + ' }');
  var parsed = rules.join('\n');

  return { hash: hash, parsed: parsed, stubs: stubs, name: name };
}

function inline(path) {
  var code = path.hub.file.code;
  var strs = path.node.quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash5.default)([].concat(_toConsumableArray(strs))); // todo - add current filename?
  var name = getName(strs.join('xxx')) || 'css';

  var stubs = path.node.quasi.expressions.map(function (x) {
    return code.substring(x.start, x.end);
  });
  var ctr = 0;

  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== stubs.length) {
      // todo - test for preceding @apply
      var applyMatch = /@apply\s*$/gm.exec(str);
      if (applyMatch) {
        arr.push('--' + name + '-' + hash + '-' + i);
      } else arr.push('var(--' + name + '-' + hash + '-' + i + ')');
    }
    return arr;
  }, []).join('').trim();

  var rules = (0, _parseCSS2.default)('.' + name + '-' + hash + ' { ' + src + ' }');
  rules = rules.map(function (rule) {
    return rule.replace(/@apply\s+--[A-Za-z0-9-_]+-([0-9]+)/gm, function (match, p1) {
      return '${x' + p1 + '}';
    });
  });
  rules = rules.map(function (rule) {
    return rule.replace(/var\(--[A-Za-z0-9-_]+-([0-9]+)\)/gm, function (match, p1) {
      return '${x' + p1 + '}';
    });
  });

  var parsed = '(' + stubs.map(function (x, i) {
    return 'x' + i;
  }).join(', ') + ') => [' + rules.map(function (x) {
    return '`' + x + '`';
  }).join(',\n') + ']';
  return { hash: hash, parsed: parsed, stubs: stubs, name: name };
}

function fragment(path) {
  var code = path.hub.file.code;
  var strs = path.node.quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash5.default)([].concat(_toConsumableArray(strs))); // todo - add current filename?
  var name = getName(strs.join('xxx')) || 'frag';

  var stubs = path.node.quasi.expressions.map(function (x) {
    return code.substring(x.start, x.end);
  });
  var ctr = 0;

  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== stubs.length) {
      // todo - test for preceding @apply
      var applyMatch = /@apply\s*$/gm.exec(str);
      if (applyMatch) {
        arr.push('--' + name + '-' + hash + '-' + i);
      } else {
        arr.push('var(--' + name + '-' + hash + '-' + i + ')');
      }
    }
    return arr;
  }, []).join('').trim();
  var rules = (0, _parseCSS2.default)('.' + name + '-' + hash + ' { --' + name + '-' + hash + ': { ' + src + ' }; }', {
    nested: false
  });
  var parsed = rules.join('\n');

  return { hash: hash, parsed: parsed, stubs: stubs, name: name };
}

function fragmentinline(path) {
  var code = path.hub.file.code;
  var strs = path.node.quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash5.default)([].concat(_toConsumableArray(strs))); // todo - add current filename?
  var name = getName(strs.join('xxx')) || 'frag';

  var stubs = path.node.quasi.expressions.map(function (x) {
    return code.substring(x.start, x.end);
  });
  var ctr = 0;

  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== stubs.length) {
      // todo - test for preceding @apply
      var applyMatch = /@apply\s*$/gm.exec(str);
      if (applyMatch) {
        arr.push('--' + name + '-' + hash + '-' + i);
      } else arr.push('var(--' + name + '-' + hash + '-' + i + ')');
    }
    return arr;
  }, []).join('').trim();

  var rules = (0, _parseCSS2.default)('.' + name + '-' + hash + ' { ' + src + ' }');
  rules = rules.map(function (rule) {
    return rule.replace(/@apply\s+--[A-Za-z0-9-_]+-([0-9]+)/gm, function (match, p1) {
      return '${x' + p1 + '}';
    });
  });
  rules = rules.map(function (rule) {
    return rule.replace(/var\(--[A-Za-z0-9-_]+-([0-9]+)\)/gm, function (match, p1) {
      return '${x' + p1 + '}';
    });
  });

  var parsed = '(' + stubs.map(function (x, i) {
    return 'x' + i;
  }).join(', ') + ') => [' + rules.map(function (x) {
    return '`' + x + '`';
  }).join(',\n') + ']';
  return { hash: hash, parsed: parsed, stubs: stubs, name: name };
}

module.exports = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      Program: {
        enter: function enter(path, state) {
          state.injected = false;
          var inserted = {};
          state.toInsert = [];
          var file = path.hub.file.opts.filename;
          state.inject = function () {
            if (!state.injected) {
              state.injected = true;

              state.toInsert.push('/* do not edit this file */');

              // let src = (state.opts.sync && !state.opts.inline) ?
              //   `import './${require('path').basename(file) + '.css'}';` :
              //   `import('./${require('path').basename(file) + '.css'}');`
              // if(!state.opts.inline){
              //   let impNode = babylon.parse(src, {sourceType: 'module', plugins: ['*']}).program.body[0]
              //   path.node.body.unshift(impNode)
              // }
            }
          };
          state.insert = function (hash, css) {
            if (!inserted[hash]) {
              inserted[hash] = true;
              state.toInsert.push(css);
            }
          };
        },
        exit: function exit(path, state) {
          var file = path.hub.file.opts.filename;

          var toWrite = state.toInsert.join('\n').trim();
          if (!state.opts.inline && state.injected && (_fs2.default.existsSync(file + '.css') ? _fs2.default.readFileSync(file + '.css', 'utf8') !== toWrite : true)) {
            if (!_fs2.default.existsSync(file + '.css')) {
              _touch2.default.sync(file + '.css');
            }

            _fs2.default.writeFileSync(file + '.css', toWrite);
          }
        }
      },
      TaggedTemplateExpression: function TaggedTemplateExpression(path, state) {
        var tag = path.node.tag;

        var code = path.hub.file.code;

        if (tag.name === 'css') {
          state.inject();

          if (state.opts.inline) {
            var newSrc = void 0;

            var _inline = inline(path),
                hash = _inline.hash,
                parsed = _inline.parsed,
                stubs = _inline.stubs,
                name = _inline.name;

            var cls = '\'' + name + '-' + hash + '\'';
            var vars = '[' + stubs.join(', ') + ']';
            newSrc = 'css(' + cls + ', ' + vars + ', ' + parsed + ')';
            path.replaceWith(babylon.parse(newSrc, { plugins: ['*'] }).program.body[0].expression);
          } else {
            var _parser = parser(path),
                _hash = _parser.hash,
                _parsed = _parser.parsed,
                _stubs = _parser.stubs,
                _name = _parser.name;

            state.insert(_hash, _parsed);
            path.replaceWith(t.callExpression(t.identifier('css'), [t.stringLiteral(_name + '-' + _hash), t.arrayExpression(path.node.quasi.expressions)]));
          }
        }

        if (tag.name === 'fragment') {
          state.inject();
          var _newSrc = void 0;
          // fragment('frag-[hash]', vars, () => [``])
          if (state.opts.inline) {
            var _fragmentinline = fragmentinline(path),
                _hash2 = _fragmentinline.hash,
                _parsed2 = _fragmentinline.parsed,
                _stubs2 = _fragmentinline.stubs,
                _name2 = _fragmentinline.name;

            var _cls = '\'' + _name2 + '-' + _hash2 + '\'';
            var _vars = '[' + _stubs2.join(', ') + ']';
            _newSrc = 'fragment(' + _cls + ', ' + _vars + ', ' + _parsed2 + ')';
          } else {
            var _fragment = fragment(path, { name: 'frag' }),
                _hash3 = _fragment.hash,
                _parsed3 = _fragment.parsed,
                _stubs3 = _fragment.stubs,
                _name3 = _fragment.name;

            state.insert(_hash3, _parsed3);
            var _cls2 = '\'' + _name3 + '-' + _hash3 + '\'';
            var _vars2 = '[' + _stubs3.join(', ') + ']';
            _newSrc = _stubs3.length > 0 ? 'fragment(' + _cls2 + ', ' + _vars2 + ')' : 'fragment(' + _cls2 + ')';
          }

          path.replaceWith(babylon.parse(_newSrc, { plugins: ['*'] }).program.body[0].expression);
        }
      }
    }
  };
};