'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sheet = undefined;
exports.flush = flush;
exports.default = css;
exports.fragment = fragment;
exports.hydrate = hydrate;

var _sheet = require('./sheet');

var _hash = require('./hash');

var _hash2 = _interopRequireDefault(_hash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var sheet = exports.sheet = new _sheet.StyleSheet();
sheet.inject();

var inserted = {};

function values(cls, vars) {
  var hash = (0, _hash2.default)([cls].concat(_toConsumableArray(vars)));
  if (inserted[hash]) {
    return 'vars-' + hash;
  }
  var fragvarcls = [];
  var src = vars.map(function (val, i) {
    return '--' + cls + '-' + i + ': ' + (/^frag-/.exec(val) ? (fragvarcls.push(val), 'var(--' + val.split(' ')[0] + ')') : val);
  }).join('; ');
  sheet.insert('.vars-' + hash + ' {' + src + '}');
  inserted[hash] = true;

  if (fragvarcls.length > 0) {
    return 'vars-' + hash + ' ' + fragvarcls.join(' ');
  }
  return 'vars-' + hash;
}

function flush() {
  sheet.flush();
  inserted = {};
  sheet.inject();
}

function css(cls, vars, content) {
  if (content) {
    var fragvarcls = [];
    // inline mode
    vars = vars.map(function (v) {
      return (/^frag-/.exec(v) ? fragments[v] : v
      );
    });
    var src = content.apply(undefined, _toConsumableArray(vars)); // returns an array
    var hash = (0, _hash2.default)(src);

    if (!inserted[hash]) {
      inserted[hash] = true;
      src.map(function (r) {
        return r.replace(new RegExp(cls, 'gm'), cls + '-' + hash);
      }).forEach(function (r) {
        return sheet.insert(r);
      });
    }
    return cls + '-' + hash;
  }
  return cls + (vars && vars.length > 0 ? ' ' + values(cls, vars) : '');
}

var fragments = {};

function fragment(frag, vars, content) {
  if (content) {
    var fragvarcls = [];
    vars = vars.map(function (v) {
      return (/^frag-/.exec(v) ? fragments[v] : v
      );
    });
    var src = content.apply(undefined, _toConsumableArray(vars)); // return array?
    if (src.length > 1) {
      throw new Error('what up!');
    }

    var hash = (0, _hash2.default)(src);
    src = src.join('');
    fragments[frag + '-' + hash] = src.substring(src.indexOf('{') + 1, src.length - 1);
    return frag + '-' + hash;
  }
  return frag + (vars && vars.length > 0 ? ' ' + values(frag, vars) : '');
}

function hydrate(ids) {
  ids.forEach(function (id) {
    return inserted[id] = true;
  });
}